/*	$Id: test-trans-close-twice.c,v 1.1 2019/09/20 12:15:58 kristaps Exp $ */
/*
 * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHORS DISCLAIM ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include "../config.h"

#if HAVE_ERR
# include <err.h>
#endif
#include <stdlib.h>
#include <string.h>

#include "../sqlbox.h"
#include "regress.h"

int
main(int argc, char *argv[])
{
	struct sqlbox		*p;
	struct sqlbox_cfg	 cfg;
	struct sqlbox_src	 srcs[] = {
		{ .fname = (char *)":memory:",
		  .mode = SQLBOX_SRC_RWC }
	};
	size_t			 srcid;
	const size_t		 tid = 1;

	memset(&cfg, 0, sizeof(struct sqlbox_cfg));
	cfg.srcs.srcs = srcs;
	cfg.srcs.srcsz = nitems(srcs);
	cfg.msg.func_short = warnx;

	if ((p = sqlbox_alloc(&cfg)) == NULL)
		errx(EXIT_FAILURE, "sqlbox_alloc");
	if (!(srcid = sqlbox_open(p, 0)))
		errx(EXIT_FAILURE, "sqlbox_open");
	if (!sqlbox_trans_immediate(p, srcid, tid))
		errx(EXIT_FAILURE, "sqlbox_trans_immediate");
	if (!sqlbox_trans_commit(p, srcid, tid))
		errx(EXIT_FAILURE, "sqlbox_trans_commit");
	if (!sqlbox_trans_commit(p, srcid, tid))
		errx(EXIT_FAILURE, "sqlbox_trans_commit");
	if (sqlbox_ping(p))
		errx(EXIT_FAILURE, "sqlbox_ping should fail");

	sqlbox_free(p);
	return EXIT_SUCCESS;
}
